# -*- coding: utf-8 -*-
#   StartUp Manager - Tool for editing settings for Grub and Usplash
#   Copyright (C) 2006-2007  Jimmy Rönnholm
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os
import sys
import shutil
import re

from subprocess import Popen, PIPE, call

class Usplash:
    """Represent the Usplash configuration."""
    def __init__(self):
        self.binary='/sbin/usplash'
        self.config_file='/etc/usplash.conf'
        self.themes_directory='/usr/lib/usplash/'
        self.alternatives_file='/etc/alternatives/usplash-artwork.so'
        if not os.path.isfile(self.binary):
            raise SystemExit, "Usplash not found."
        if not os.path.isdir(self.themes_directory):
            os.mkdir(self.themes_directory)
        if not os.path.isfile(self.config_file):
            self.set_resolution(["1024", "768"])
        self.themes=[]
        self.default_theme=""
        self.update_initramfs=False
        self.__check_themes()

    def __check_themes(self):
        """Check for installed themes."""
        self.themes=[]
        current_usplash=""
        name=self.themes_directory+"usplash-artwork.so"
        if os.path.islink(name):
            current_usplash_path=os.path.realpath(name)
            slash=current_usplash_path.rfind("/")
            if slash != -1:
                path=current_usplash_path[:slash+1]
                if path != self.themes_directory:
                    self.add_theme(current_usplash_path)
                current_usplash=current_usplash_path[slash+1:]

        usplash_files=os.listdir(self.themes_directory)
        for usplash_file in usplash_files:
            end=usplash_file[usplash_file.rfind("."):]
            if end == ".so" and usplash_file != "usplash-artwork.so":
                self.themes.append(usplash_file[:-3])
                if usplash_file == current_usplash:
                    self.default_theme=usplash_file[:-3]

    def set_resolution(self, resolution):
        """Change /etc/usplash.conf based on resolution[0]=xres and [1]=yres."""
        xres=resolution[0]
        yres=resolution[1]
        output_file=open(self.config_file, 'w')
        output_file.write("# Usplash configuration file\n")
        output_file.write("xres="+xres+"\n")
        output_file.write("yres="+yres+"\n")
        output_file.close()
        self.update_initramfs=True

    def get_active_theme(self):
        return self.default_theme

    def set_active_theme(self, name):
        filename=name+".so"
        for usplash_file in os.listdir(self.themes_directory):
            if filename == usplash_file:
                os.system("ln -sf "+self.alternatives_file+" "+self.themes_directory+"usplash-artwork.so")
                command= '/usr/sbin/update-alternatives --set usplash-artwork.so /usr/lib/usplash/'+filename
                os.system(command)
                self.default_theme=name
                self.update_initramfs=True

    def get_all_themes(self):
        return self.themes

    def add_theme(self, path):
        filename=path[path.rfind("/")+1:]
        if filename[-3:] == ".so" and filename != "usplash-artwork.so":
            shutil.copy(path, self.themes_directory)
            command='/usr/sbin/update-alternatives --install /usr/lib/usplash/usplash-artwork.so '+\
            'usplash-artwork.so '+self.themes_directory+filename+' 10'
            os.system(command)
            self.__check_themes()

    def remove_theme(self, filename):
        filename=filename+".so"
        if filename != "usplash-artwork.so":
            filename=self.themes_directory+filename
            os.remove(filename)
            self.__check_themes()

    def do_shutdown_tasks(self):
        if self.update_initramfs:
            os.system('update-initramfs -k $(uname -r) -u')

class Grub:
    """Represent the Grub configuration."""
    def __strip_line(self, line, start_slice, end_slice, remove_sharp):
        """Return slice of line with leading and trailing spaces removed, remove leading # if remove_sharp is True"""
        line=line.strip()
        if remove_sharp and len(line)>0 and line[0] == "#":
            line=line[1:]
            line=line.strip()
        elif len(line)>0 and line[0] == "#":
            line=line[1:]
            line=line.strip()
            line="#"+line
        line=line[start_slice:end_slice]
        line=line.strip()
        return line

    def __evaluate(self, line):
        """Extract info from line to class variables."""
        number_filter=re.compile('[0-9]+')
        if self.__strip_line(line, None, 7, True) == "default":
            if self.__strip_line(line, 7, None, True).find("saved") != -1:
                self.default_boot=-1
            else:
                match=number_filter.search(self.__strip_line(line, 7, None, True))
                if match:
                    self.default_boot=int(match.group())

        if self.__strip_line(line, None, 5, False) == "title":
            self.titles.append(self.__strip_line(line, 5, None, False))

        if self.__strip_line(line, None, 7, True) == "timeout":
            match=number_filter.search(self.__strip_line(line, 7, None, True))
            if match:
                self.timeout=int(match.group())
            if self.__strip_line(line, None, 7, False) == "timeout":
                self.use_timeout=True

        if self.__strip_line(line, None, 10, False) == "hiddenmenu":
            self.show_menu=False

        if self.__strip_line(line, None, 10, True) == "defoptions":
            if line.find("quiet") == -1:
                self.show_text=True

            if line.find("splash") == -1:
                self.show_splash=False

            if line.find("vga=") != -1:
                place=line.find("vga=")+4
                if line[place:place+2] == "0x":
                    self.vga=int(line[place:place+5],16)
                else:
                    self.vga=int(line[place:place+3])

        if self.__strip_line(line, None, 5, True) == "color":
            hidden=line.strip()[:1]
            if hidden != "#":
                self.use_colors=True
            color_line=line.replace("#","").replace("color","").replace("=","").strip()
            first_slash=color_line.find("/")
            last_slash=color_line.rfind("/")
            if first_slash == last_slash:
                if color_line.find("blink-") != -1:
                    self.normal_blink=True
                    blinkplace=color_line.find("blink-")
                    color_line=color_line[:blinkplace]+color_line[blinkplace+6:]
                first_slash=color_line.find("/")
                last_slash=color_line.rfind("/")
                self.normal_foreground=color_line[:first_slash]
                self.normal_background=color_line[first_slash+1:]
                self.highlighted_foreground="white"
                self.highlighted_background="blue"
            else:
                first_blinkplace=color_line.find("blink-")
                last_blinkplace=color_line.rfind("blink-")
                space=color_line.find(" ")
                if first_blinkplace != -1:
                    if first_blinkplace != last_blinkplace:
                        self.normal_blink=True
                        self.highlighted_blink=True
                        color_line=color_line[:first_blinkplace]+color_line[first_blinkplace+6:last_blinkplace]+color_line[last_blinkplace+6:]
                    else:
                        if first_blinkplace > space:
                            self.highlighted_blink=True
                        else:
                            self.normal_blink=True
                        color_line=color_line[:first_blinkplace]+color_line[first_blinkplace+6:]
                space=color_line.find(" ")
                first_slash=color_line.find("/")
                last_slash=color_line.rfind("/")
                self.normal_foreground=color_line[:first_slash]
                self.normal_background=color_line[first_slash+1:space]
                self.highlighted_foreground=color_line[space+1:last_slash]
                self.highlighted_background=color_line[last_slash+1:]

        if self.__strip_line(line, None, 5, True) == "groot":
            self.groot=line[line.find("hd")-1:line.find("hd")+6]

        if self.__strip_line(line, None, 11, False) == "splashimage":
            self.use_image=True

        if self.__strip_line(line, None, 8, False) == "password":
            self.password_protect=True

        if self.__strip_line(line, None, 15, True) == "lockalternative":
            if line.find("true") != -1:
                self.protect_rescuemode=True

        if self.__strip_line(line, None, 7, True) == "lockold":
            if line.find("true") != -1:
                self.protect_oldmode=True

        if self.__strip_line(line, None, 7, True) == "howmany":
            if line.find("all") != -1:
                self.kernel_limit="all"
            else:
                match=number_filter.search(self.__strip_line(line, 7, None, True))
                if match:
                    self.kernel_limit=int(match.group())

        if self.__strip_line(line, None, 9, True) == "memtest86":
            if line.find("true") == -1:
                self.memtest=False

        if self.__strip_line(line, None, 18, True) == "updatedefaultentry":
            if line.find("true") == -1:
                self.update_default=False

        if self.__strip_line(line, None, 11, True) == "alternative":
            if line.find("true") == -1:
                self.create_alternative=False

    def __read_config(self):
        """Read grub configuration file, pass lines to __evaluate()."""
        grub_lines=self.__get_lines_of_file(self.menu)
        for line in grub_lines:
            self.__evaluate(line)
        self.__find_separator()

    def __check_themes(self):
        """Check for installed themes, send info to class variables."""
        self.grub_images=[]
        if self.__get_line_number("splash") != -1:
            grub_lines=self.__get_lines_of_file(self.menu)
            location=self.__get_line_number("splash")
            splash_line=grub_lines[location].strip()
            splash_path=splash_line.find(self.groot+self.mounted_splashdir)
            if splash_path != -1:
                file_name=splash_line[splash_path+31:]
                suffix=file_name.find(".xpm.gz")
                if suffix != -1:
                    name=file_name[:suffix]
                    for dirfile in os.listdir(self.mounted_splashdir):
                        if file_name == dirfile:
                            self.default_image=name

        grub_files=os.listdir(self.mounted_splashdir)
        for grub_file in grub_files:
            end=grub_file[grub_file[:-3].rfind("."):]
            if end == ".xpm.gz":
                self.grub_images.append(grub_file[:-7])

    def __update_grub_colors(self):
        """Update Grub color line based on the class variables."""
        first_blink=""
        last_blink=""
        if self.normal_blink:
            first_blink="blink-"
        if self.highlighted_blink:
            last_blink="blink-"
        color_line="color "+first_blink+self.normal_foreground+"/"+self.normal_background+" "\
        +last_blink+self.highlighted_foreground+"/"+self.highlighted_background+"\n"
        if not self.use_colors:
            color_line="#"+color_line
        self.__change_config("color", None, color_line)

    def __get_line_number(self, identifier):
        """Return the number of the line that begin with the string identifier."""
        lines=self.__get_lines_of_file(self.menu)
        length=len(identifier)
        tracker=0
        for line in lines:
            if line.strip()[:1] == "#":
                line=line.strip()[1:]
            line=line.strip()[:length].strip()
            if line == identifier:
                return tracker
            tracker+=1
        return -1

    def __change_config(self, identifier, old_value, new_value):
        """Change the configuration file based on the passed values."""
        line_number=self.__get_line_number(identifier)
        lines=self.__get_lines_of_file(self.menu)
        if line_number == -1:
            line_number=0
            lines.insert(0,"\n")
        line=lines[line_number]
        if old_value == None:
            lines[line_number]=new_value
        elif old_value == "vga=":
            place=line.find(old_value)
            end=line.find(" ", place)
            if place != -1:
                if end != -1:
                    line=line[:place]+new_value+line[end:]
                else:
                    line=line[:place]+new_value+"\n"
            else:
                line=line[:-1]+" "+new_value+"\n"
            lines[line_number]=line
        else:
            if old_value[0] != "#" and line.find(" "+old_value) != -1:
                old_value=" "+old_value

            if old_value[0] == "#" and line.find("# "+old_value[1:]) != -1:
                old_value="# "+old_value[1:]

            line=line.replace(old_value, new_value)
            lines[line_number]=line

        output_file=open(self.menu, 'w')
        for out_line in lines:
            output_file.write(out_line)
        output_file.close()

    def __get_lines_of_file(self, filename):
        """Read file filename, return list of lines in file."""
        try:
            input_file=open(filename, 'r')
            lines=input_file.readlines()
            input_file.close()
            return lines
        except IOError:
            raise SystemExit, "File "+filename+" does not exist."

    def __find_separator(self):
        """Find the separator line, update variable."""
        title_number=0
        for title in self.titles:
            if title == "Other operating systems:":
                self.separator=title_number
            title_number+=1

    def __fix_menu_lst_bug(self):
        """Take care of an annoying thing in menu.lst."""
        grub_lines=self.__get_lines_of_file(self.menu)
        password_lines=[]
        for line in grub_lines:
            line.strip()
            if line[0] == "#":
                line=line[1:].strip()
            if line[:8] == "password":
                password_lines.append(line)
        if len(password_lines) == 2:
            self.__change_config(password_lines[0], None, "## "+password_lines[0]+"\n")

    def __has_separate_boot_partition(self):
        for line in self.__get_lines_of_file(self.fstab):
            if line.find("/boot")!=-1:
                return True
        return False

    def has_update_grub(self):
        return self.has_advanced_options

    def get_default_boot(self):
        default_boot=self.default_boot
        if default_boot != -1:
            if (default_boot>self.separator) and (self.separator != -1):
                self.default_boot-=1
        return default_boot

    def get_titles(self):
        return self.titles

    def set_default_boot(self, default):
        self.default_boot=default
        if default == -1:
            self.__change_config("default", None, "default		"+"saved\n")
        else:
            if default>=self.separator and self.separator != -1:
                default+=1
            self.__change_config("default", None, "default		"+str(default)+"\n")

    def get_timeout(self):
        return self.timeout

    def get_timeout_used(self):
        return self.use_timeout

    def set_timeout(self, active, timeout):
        self.timeout=timeout
        self.use_timeout=active
        if active:
            self.__change_config("timeout", None, "timeout		"+str(timeout)+"\n")
        else:
            self.__change_config("timeout", None, "#timeout		"+str(timeout)+"\n")

    def get_vga(self):
        return self.vga

    def set_resolution(self, number):
        dic={
        "00" : "769" ,
        "10" : "771" ,
        "20" : "773" ,
        "30" : "775" ,
        "40" : "796" ,
        "01" : "758" ,
        "11" : "788" ,
        "21" : "791" ,
        "31" : "794" ,
        "41" : "798" ,
        "02" : "786" ,
        "12" : "789" ,
        "22" : "792" ,
        "32" : "795" ,
        "42" : "799" 
        }
        vga=dic[number]
        self.vga=vga
        self.__change_config("defoptions", "vga=", "vga="+vga)
        self.update_grub=True

    def get_boot_text_visible(self):   
        return self.show_text

    def set_boot_text_visible(self, active):
        self.show_text=active
        if active:
            self.__change_config( "defoptions", "quiet", "")
        else:
            self.__change_config( "defoptions", "\n", " quiet\n")
        self.update_grub=True

    def get_menu_visible(self):
        return self.show_menu

    def set_menu_visible(self, active):
        self.show_menu=active
        if active:
            self.__change_config("hiddenmenu", None, "#hiddenmenu\n")
        else:
            self.__change_config("hiddenmenu", None, "hiddenmenu\n")

    def get_use_colors(self):
        return self.use_colors

    def set_use_colors(self, active):
        self.use_colors=active
        self.__update_grub_colors()

    def get_color_normal_bg(self):
        return self.normal_background

    def get_color_highlighted_bg(self):
        return self.highlighted_background

    def get_color_normal_fg(self):
        return self.normal_foreground

    def get_color_highlighted_fg(self):
        return self.highlighted_foreground

    def get_color_blink_normal(self):
        return self.normal_blink

    def get_color_blink_highlight(self):
        return  self.highlighted_blink

    def set_color_normal_bg(self, color):
        self.normal_background=color
        self.__update_grub_colors()

    def set_color_highlighted_bg(self, color):
        self.highlighted_background=color
        self.__update_grub_colors()

    def set_color_normal_fg(self, color):
        self.normal_foreground=color
        self.__update_grub_colors()

    def set_color_highlighted_fg(self, color):
        self.highlighted_foreground=color
        self.__update_grub_colors()

    def set_color_blink_normal(self, active):
        self.normal_blink=active
        self.__update_grub_colors()

    def set_color_blink_highlight(self, active):
        self.highlighted_blink=active
        self.__update_grub_colors()

    def get_splash_visible(self):
        return self.use_image

    def get_splash(self):
        return self.default_image

    def set_splash(self, active, image_name):
        self.default_image=image_name
        self.show_splash=active
        image_name=image_name+".xpm.gz"
        text="\n#A splash image for the menu\n"
        if self.__get_line_number("splashimage") != -1:
            if active:
                text="splashimage="+self.groot+self.splashdir
            else:
                text="#splashimage="+self.groot+self.splashdir
            self.__change_config("splashimage", None, text+image_name+"\n")
        else:
            for splash_file in os.listdir(self.mounted_splashdir):
                if image_name == splash_file:
                    if active:
                        text=text+"splashimage="+self.groot+self.splashdir
                    else:
                        text=text+"#splashimage="+self.groot+self.splashdir
                    self.__change_config("color", "\n", "\n"+text+image_name+"\n")

    def get_images(self):
        return self.grub_images

    def add_image(self, filename):
        pathless=filename[filename.rfind("/")+1:]
        end=filename.rfind(".")
        if not end == -1:
            if filename[end:] == ".gz":
                end=filename[:-3].rfind(".")
            if filename[end:] == ".xpm.gz":
                shutil.copy(filename, self.mounted_splashdir)
                self.__check_themes()
            elif filename[end:] == ".xpm":
                os.system("gzip -c "+filename+" > "+self.mounted_splashdir+pathless+".gz")
                self.__check_themes()
            elif filename[end:] == ".png":
                end=pathless.rfind(".")
                os.system("convert "+filename+" -resize 640x480 -colors 14 "+pathless[:end]+".xpm")
                os.system("gzip -c "+pathless[:end]+".xpm"+" > "+self.mounted_splashdir+pathless[:end]+".xpm.gz")
                os.system("rm -f "+pathless[:end]+".xpm")
                self.__check_themes()

    def remove_image(self, filename):
        filename=self.mounted_splashdir+filename+".xpm.gz"
        os.remove(filename)
        self.__check_themes()

    def get_splash_active(self):
        return self.show_splash

    def set_splash_active(self, active):
        self.show_splash=active
        if active:
            self.__change_config("defoptions", "\n", " splash\n")
        else:
            self.__change_config("defoptions", "splash", "")
        self.update_grub=True

    def get_password_protection(self):
        return self.password_protect

    def set_password_protection(self, active):
        self.password_protect=active
        if active:
            self.__change_config("password", "#password", "password")
        else:
            self.__change_config("password", "password", "#password")

    def get_protect_rescuemode(self):
        return self.protect_rescuemode

    def set_protect_rescuemode(self, active):
        self.protect_rescuemode=active
        if active:
            self.__change_config("lockalternative", None, "# lockalternative=true\n")
        else:
            self.__change_config("lockalternative", None, "# lockalternative=false\n")
        self.update_grub=True

    def get_protect_oldmode(self):
        return self.protect_oldmode

    def set_protect_oldmode(self, active):
        self.protect_oldmode=active
        if active:
            self.__change_config("lockold", None, "# lockold=true\n")
        else:
            self.__change_config("lockold", None, "# lockold=false\n")
        self.update_grub=True

    def update_password(self, password, active):
        if len(password) > 3:
            communication=Popen(['grub-md5-crypt'], stdin=PIPE, stdout=PIPE, close_fds=True)
            password=communication.communicate(password+"\n"+password+"\n")[0].rsplit('\n')[2]
            password="password --md5 "+password
            if not active:
                password="#"+password

            self.__change_config("password", None, password)

        else:
            if len(password) < 4:
                raise SystemExit, "Password must be at least 4 characters"
            else:
                raise SystemExit, "Passwords do not match"

    def get_limit_kernel(self):
        return self.kernel_limit

    def set_kernel_limit(self, active, number):
        if not active:
            number="all"
        self.__change_config("howmany", None, "# howmany="+str(number)+"\n")
        self.kernel_limit=number
        self.update_grub=True

    def get_memtest(self):
        return self.memtest

    def set_memtest(self, active):
        self.memtest=active
        if active:
            self.__change_config("memtest86", None, "# memtest86=true\n")
        else:
            self.__change_config("memtest86", None, "# memtest86=false\n")
        self.update_grub=True

    def get_update_default(self):
        return self.update_default

    def set_update_default(self, active):
        self.update_default=active
        if active:
            self.__change_config("updatedefaultentry", None, "# updatedefaultentry=true\n")
        else:
            self.__change_config("updatedefaultentry", None, "# updatedefaultentry=false\n")
        self.update_grub=True

    def get_create_alternative(self):
        return self.create_alternative

    def set_create_alternative(self, active):
        self.create_alternative=active
        if active:
            self.__change_config("alternative", None, "# alternative=true\n")
        else:
            self.__change_config("alternative", None, "# alternative=false\n")
        self.update_grub=True

    def format_floppy(self):
        try:
            retcode = call("/sbin/mke2fs /dev/fd0", shell=True)
            if retcode < 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode

            return retcode
        except OSError, e:
            print >>sys.stderr, "Execution failed:", e
            return 3

    def make_floppy(self):
        dir_exist=True
        if not os.path.isdir('/mnt/floppysum'):
            os.mkdir('/mnt/floppysum')
            dir_exist=False
        try:
            retcode = call("mount -t ext2 /dev/fd0 /mnt/floppysum", shell=True)
            if not retcode == 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode
                return retcode
            retcode = call("/sbin/grub-install --root-directory=/mnt/floppysum '(fd0)'", shell=True)
            if not retcode == 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode
                return retcode
            retcode = call("cp "+self.menu+"  /mnt/floppysum/"+self.menu, shell=True)
            if not retcode == 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode
                return retcode
            retcode = call("umount /mnt/floppysum", shell=True)
            if not retcode == 0:
                print >>sys.stderr, "Child was terminated by signal", -retcode
                return retcode
        except OSError, e:
            print >>sys.stderr, "Execution failed:", e
            return 1
        if not dir_exist:
            os.rmdir('/mnt/floppysum')
        return 0

    def do_shutdown_tasks(self):
        if self.update_grub:
            os.system(self.update_grub_command)

    def get_automagic_line(self):
        return self.has_automagic_line

    def continue_init(self):
        if not os.path.isfile(self.update_grub_command):
            self.update_grub_command='/sbin/update-grub'

        if not os.path.isfile(self.update_grub_command):
            self.has_advanced_options=False

        if self.has_advanced_options==True:
            communication=Popen([self.update_grub_command], stdin=PIPE, close_fds=True)
            communication.communicate("y\n")

        self.menu='/boot/grub/menu.lst'

        if not os.path.isfile(self.menu):#We do this again, in case update-grub has created a new file.
            self.menu='/boot/grub/grub.conf'
            if os.path.isfile(self.menu):
                if not os.path.isfile(self.menu+".backup"):
                    shutil.copy(self.menu, self.menu+".backup")

        if not os.path.isfile(self.menu):
            self.menu='/etc/grub.conf'
            if os.path.isfile(self.menu):
                if not os.path.isfile(self.menu+".backup"):
                    shutil.copy(self.menu, self.menu+".backup")

        if not os.path.isfile(self.menu):
            raise SystemExit, "Fatal error. Can not find the bootloader configuration file"

        if not os.path.isfile(self.menu+".backup"):
            shutil.copy(self.menu, self.menu+".backup")

        if not os.path.isdir(self.mounted_splashdir):
            os.mkdir(self.mounted_splashdir)

        self.__fix_menu_lst_bug()
        os.chdir(self.workdir)
        self.__read_config()
        self.__check_themes()

    def __init__(self):
        self.menu='/boot/grub/menu.lst'
        self.workdir='/tmp/'
        self.fstab='/etc/fstab'
        self.mounted_splashdir='/boot/grub/splashimages/'
        if self.__has_separate_boot_partition():
            self.splashdir='/grub/splashimages/'
        else:
            self.splashdir='/boot/grub/splashimages/'
        self.update_grub_command='/usr/sbin/update-grub'
        self.titles=[]
        self.default_boot=0
        self.separator=-1
        self.use_timeout=False
        self.timeout=0
        self.vga=769
        self.show_text=False
        self.show_menu=True
        self.use_colors=False
        self.normal_background="black"
        self.highlighted_background="black"
        self.normal_foreground="black"
        self.highlighted_foreground="black"
        self.normal_blink=False
        self.highlighted_blink=False
        self.groot=""
        self.use_image=False
        self.grub_images=[]
        self.default_image=""
        self.show_splash=True
        self.password_protect=False
        self.protect_rescuemode=False
        self.protect_oldmode=False
        self.kernel_limit="all"
        self.memtest=True
        self.update_default=True
        self.update_grub=False
        self.create_alternative=True
        self.has_advanced_options=True
        self.has_automagic_line=False

        if not os.path.isfile(self.menu):
            self.menu='/boot/grub/grub.conf'
            if os.path.isfile(self.menu):
                if not os.path.isfile(self.menu+".backup"):
                    shutil.copy(self.menu, self.menu+".backup")

        if not os.path.isfile(self.menu):
            self.menu='/etc/grub.conf'
            if os.path.isfile(self.menu):
                if not os.path.isfile(self.menu+".backup"):
                    shutil.copy(self.menu, self.menu+".backup")

        if os.path.isfile(self.menu):
            grub_lines=self.__get_lines_of_file(self.menu)
            for line in grub_lines:
                if self.__strip_line(line, None, 37, False) == "### END DEBIAN AUTOMAGIC KERNELS LIST":
                    self.has_automagic_line=True
        #We break the initialization here and resume it from the frontend code later with the continue_init function. Ugly hack, I know. Sorry for that :(
